/***********************************************************
 * $Id: InterceptedMethodInvocation.java 333 2010-08-31 21:27:18Z util $
 * 
 * AOP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 31.08.2010
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.aop;

import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Method;

import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;

/**
 * An implementation of {@link MethodInvocation}, which is intercepted by a
 * {@link MethodInterceptor}.
 */
public class InterceptedMethodInvocation implements MethodInvocation
{
    private final MethodInvocation target;
    private final MethodInterceptor interceptor;

    /**
     * @param target
     * @param interceptor
     */
    public InterceptedMethodInvocation(MethodInvocation target,
                                       MethodInterceptor interceptor)
    {
        super();
        this.target = target;
        this.interceptor = interceptor;
    }

    /* (non-Javadoc)
     * @see org.aopalliance.intercept.Invocation#getArguments()
     */
    public Object[] getArguments()
    {
        return this.target.getArguments();
    }

    /* (non-Javadoc)
     * @see org.aopalliance.intercept.Joinpoint#proceed()
     */
    public Object proceed() throws Throwable
    {
        return this.interceptor.invoke(this.target);
    }

    /* (non-Javadoc)
     * @see org.aopalliance.intercept.Joinpoint#getThis()
     */
    public Object getThis()
    {
        return this.target.getThis();
    }

    /* (non-Javadoc)
     * @see org.aopalliance.intercept.Joinpoint#getStaticPart()
     */
    public AccessibleObject getStaticPart()
    {
        return this.target.getStaticPart();
    }

    /* (non-Javadoc)
     * @see org.aopalliance.intercept.MethodInvocation#getMethod()
     */
    public Method getMethod()
    {
        return this.target.getMethod();
    }

}
