/***********************************************************
 * $Id: ReflectiveMethodInvocation.java 333 2010-08-31 21:27:18Z util $
 * 
 * AOP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 2010-08-31
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.aop;

import java.lang.reflect.AccessibleObject;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.aopalliance.intercept.MethodInvocation;

/**
 * An implementation of {@link MethodInvocation}, which operates on an ordinary
 * Java Object.
 */
public class ReflectiveMethodInvocation implements MethodInvocation
{
    private final Object target;
    private final Method method;
    private final Object[] arguments;

    /**
     * @param target The object on which to invoke the given method.
     * @param method The method to call.
     * @param arguments The arguments to the method.
     */
    public ReflectiveMethodInvocation(Object target,
                                      Method method,
                                      Object[] arguments)
    {
        super();
        this.target = target;
        this.method = method;
        this.arguments = arguments;
    }

    /* (non-Javadoc)
     * @see org.aopalliance.intercept.Invocation#getArguments()
     */
    public Object[] getArguments()
    {
        return this.arguments;
    }

    /* (non-Javadoc)
     * @see org.aopalliance.intercept.Joinpoint#proceed()
     */
    public Object proceed() throws Throwable
    {
        try {
            return this.method.invoke(this.target,this.arguments);
        }
        catch(InvocationTargetException ite) {
            throw ite.getCause();
        }
        catch(IllegalAccessException iae) {
            throw new SecurityException("Access was denied to method ["+this.method+"]: "+iae.getMessage());
        }
    }

    /* (non-Javadoc)
     * @see org.aopalliance.intercept.Joinpoint#getThis()
     */
    public Object getThis()
    {
        return this.target;
    }

    /* (non-Javadoc)
     * @see org.aopalliance.intercept.Joinpoint#getStaticPart()
     */
    public AccessibleObject getStaticPart()
    {
        return this.method;
    }

    /* (non-Javadoc)
     * @see org.aopalliance.intercept.MethodInvocation#getMethod()
     */
    public Method getMethod()
    {
        return this.method;
    }

}
