/***********************************************************
 * $Id: GenericJoinDAO.java 418 2011-01-01 22:28:15Z util $
 * 
 * AOP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 2006-03-25
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/


package org.clazzes.util.aop.generic;

import java.io.Serializable;
import java.util.List;

/**
 * This interface encapsulates DAOs for types <tt>T</tt> which reference
 * a base type <tt>B</tt> through a foreign key.
 *
 * @param <T> The type, which is retrieved by this DAO.
 * @param <B> The base type to which the type T is joined.
 */
public interface GenericJoinDAO<T,B> extends GenericDAO<T>
{
    /**
     * @return The class object for the persistent base class <code>B</code>. 
     */
    Class<B> getPersistentBaseClass();
 
    /**
	 * @param bId The database Id of the base object with the type<tt>B</tt>.
	 * @return A list of all objects of type <tt>T</tt> joined to an object of type
	 *         <tt>B</tt> from the DB.
	 */
	List<T> getAllJoined(Serializable bId);
}
