/***********************************************************
 * $Id: JdbcStatementAction.java 340 2010-09-02 07:12:29Z util $
 * 
 * AOP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 01.09.2010
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.aop.jdbc;

import java.sql.Statement;

import org.clazzes.util.aop.DAOException;

/**
 * This interface encapsulates JDBC code, which is executed as part of a
 * DAO. This way, the given statement can be closed in a finally clause
 * and any SQLException may be encapsulated by a {@link DAOException}.
 * 
 * If your action does not return a value, parameterize it with {@link Void}
 * and return <code>null</code> from your implementation.
 */
public interface JdbcStatementAction<T>
{
    /**
     * Run a piece of JDBC code inside a context, where a statement has
     * been set up. 
     * 
     * @param statement A JDBC statement, which will be closed upon the finalization of
     *                  this method.
     * @return The result of the DAO operation or <code>null</code>.
     * @throws Exception Upon errors. This exception will be wrapped by a {@link DAOException}.
     */
    T perform(Statement statement) throws Exception;
}
